# pylint: disable=W,C,R

from dataclasses import dataclass, field
from enum import Enum
from typing import List, Union

from dataclasses_json import dataclass_json


@dataclass
class Field:
    """Model output """

    value: str
    confidence: float
    parsed_value: Union[str, None] = None
    bbox: Union[List[int], None] = None


@dataclass
class RawInvoice:
    """RAW model output """

    kid: List[Field] = field(default_factory=list)  # can this be an empty string, and then set the confidence to 0?
    # this way, we won't have to deal with None values.
    # Also, is_valid=0 is the same as confidence=0, so we could just set the confidence really low and make the user
    # of the api determine if it is invalid based on the confidence.
    date: List[Field] = field(default_factory=list)
    due_date: List[Field] = field(default_factory=list)
    principal_amount: List[Field] = field(default_factory=list)
    customer_number: List[Field] = field(default_factory=list)
    invoice_account_number: List[Field] = field(default_factory=list)
    invoice_number: List[Field] = field(default_factory=list)


@dataclass
class RawBankvoucher:
    """RAW model output of bankvoucher"""

    payer: List[Field] = field(default_factory=list)
    creditor: List[Field] = field(default_factory=list)
    from_account_number: List[Field] = field(default_factory=list)
    to_account_number: List[Field] = field(default_factory=list)
    message: List[Field] = field(default_factory=list)
    entrybox: List[Field] = field(default_factory=list)
    principal_amount: List[Field] = field(default_factory=list)
    date: List[Field] = field(default_factory=list)


@dataclass_json
@dataclass
class BankVoucher(RawBankvoucher):
    pass


@dataclass_json
@dataclass
class Invoice:
    """RAW model output """

    kid: Union[Field, None] = None
    date: Union[Field, None] = None
    due_date: Union[Field, None] = None
    principal_amount: Union[Field, None] = None
    customer_number: Union[Field, None] = None
    invoice_account_number: Union[Field, None] = None
    invoice_number: Union[Field, None] = None


class FieldName(str, Enum):
    # invoice
    kid = "kid"
    invoice_account_number = "invoicingAccountNumber"
    due_date = "dueDate"
    date = "date"
    principal_amount = "principalAmount"
    invoice_number = "invoiceNumber"
    customer_number = "customerNumber"

    # bankvoucher
    payer = "payer"
    creditor = "creditor"
    from_account_number = "fromAccountNumber"
    to_account_number = "toAccountNumber"
    message = "message"
    entrybox = "entryBox"  # used to sepearate entries
