import glob
import json
import os
from dataclasses import dataclass
from typing import Any, Dict, Iterator, Tuple

from dependency_injector.wiring import Provide, inject

from ..containers.core_container import ContainerRef
from ..pdfreader.pdf_reader import PdfReader


@dataclass
class DocumentData:
    text: str
    json: Dict[str, Any]
    file: str


class DatasetLoader:
    @inject
    def __init__(self, root_path: str, pdf_reader: PdfReader = Provide[ContainerRef.pdf_reader]) -> None:
        assert root_path is not None, root_path
        self.root_path = root_path
        self.pdf_reader = pdf_reader
        self.file_names = list(
            map(
                lambda x: os.path.basename(x).split(".")[0],
                glob.glob(os.path.join(root_path, "pdfs/*.pdf")),
            )
        )

    def load(self, basename: str) -> Tuple[str, Dict[str, Any]]:
        pdf_path = os.path.join(self.root_path, "pdfs/{}.pdf".format(basename))
        json_path = os.path.join(self.root_path, "jsons/{}.json".format(basename))
        content = self.pdf_reader.read(pdf_path)
        with open(json_path, "r") as file:
            return content, json.loads(file.read())

    def __iter__(self) -> Iterator[DocumentData]:
        return iter([DocumentData(*self.load(base_name), base_name) for base_name in self.file_names])
