from dataclasses import asdict
from typing import Any

from yolov3.default import get_data_config, get_model_prefix

from ..api.bankvoucher import BankVoucher
from ..api.field import DateField, StringField
from ..api.invoice import Invoice
from .model import Model


class ModelYolo(Model):
    def __init__(self):
        pass

    def predict_invoice(self, image_file: Any) -> Invoice:
        return self.predict(image_file, is_bank_voucher=False)

    def predict_bankvoucher(self, image_file: Any) -> BankVoucher:
        return self.predict(image_file, is_bank_voucher=True)

    def predict(self, image_file, is_bank_voucher):
        from pdfinvoicereader.parse_raw_document import parse_document
        from yolov3.detect_interface import Detect

        _, clean_document = parse_document(
            Detect(
                data_config=get_data_config(is_bank_voucher), prefix=get_model_prefix(is_bank_voucher)
            ).get_document_from_image(image_file, debug=True)
        )

        return self._convert_to_graphql(clean_document)

    def _convert_to_graphql(self, bank_document):
        from ..parse_raw_document import first_or_none

        # TODO : fix it so isinstance work on dataclass
        if bank_document[0].__class__.__name__ == "BankVoucher":
            parsed = []
            for document in bank_document:
                key_values = {}
                for key, value in asdict(document).items():
                    print(key, value)
                    key_values[key] = self._resolve_field(first_or_none(value))
                parsed.append(BankVoucher(**key_values))
            return parsed
        parsed = []
        for document in bank_document:
            key_values = {}
            for key, value in asdict(document).items():
                print(key, value)
                key_values[key] = self._resolve_field(first_or_none(value))
            parsed.append(Invoice(**key_values))
        return parsed

    def _resolve_field(self, first_or_none_value):
        if first_or_none_value is None:
            return None
        value = first_or_none_value["value"]
        confidence = first_or_none_value["confidence"]
        if isinstance(value, str):
            return StringField(value=value, confidence=confidence)
        return DateField(value=value, confidence=confidence)
