from typing import List

import numpy as np

from ..pdf_reader import BboxText


def merge(raw_document, pdf_text_bbox: List[BboxText], model_page_size=None):
    for field_key in raw_document.__dataclass_fields__:
        fields = getattr(raw_document, field_key)
        if fields is None:
            continue
        for index, field in enumerate(fields):
            field_text = field.value
            field_bbox = field.bbox

            if model_page_size is not None:
                for i in pdf_text_bbox:
                    delta_x = model_page_size / i.page_width
                    delta_y = model_page_size / i.page_height
                    i.bbox[0] *= delta_x
                    i.bbox[1] *= delta_y
                    i.bbox[2] *= delta_x
                    i.bbox[3] *= delta_y

            for text_bbox in pdf_text_bbox:
                bbox = text_bbox.bbox
                bbox_text = text_bbox.text
                if (
                    bbox[0] <= field_bbox[0]
                    and bbox[1] <= field_bbox[1]
                    and field_bbox[2] <= bbox[2]
                    and field_bbox[3] <= bbox[3]
                ):
                    if edit_distance(field_text, bbox_text) <= 3:
                        fields[index].value = bbox_text
        setattr(raw_document, field_key, fields)
    return raw_document


# based off https://stackabuse.com/levenshtein-distance-and-text-similarity-in-python/
def edit_distance(seq1, seq2):
    size_x = len(seq1) + 1
    size_y = len(seq2) + 1
    matrix = np.zeros((size_x, size_y))
    for x in range(size_x):
        matrix[x, 0] = x
    for y in range(size_y):
        matrix[0, y] = y

    for x in range(1, size_x):
        for y in range(1, size_y):
            if seq1[x - 1] == seq2[y - 1]:
                matrix[x, y] = min(matrix[x - 1, y] + 1, matrix[x - 1, y - 1], matrix[x, y - 1] + 1)
            else:
                matrix[x, y] = min(matrix[x - 1, y] + 1, matrix[x - 1, y - 1] + 1, matrix[x, y - 1] + 1)
    return matrix[size_x - 1, size_y - 1]
