import unittest

from pdf_invoice_reader_common.interface import Field, RawInvoice

from ..pdf_reader import BboxText
from ..pdf_reader_mock import PdfReaderMock
from .pdf_ocr_merger import merge


class PdfOcrMergerTest(unittest.TestCase):
    def test_merge(self):
        basic_invoice = RawInvoice(kid=[Field(value="est", confidence=0, bbox=[10, 10, 20, 20])])
        pdf_reader_mock = PdfReaderMock()
        file_name = "example"
        pdf_reader_mock.add_file(file_name, "test", [BboxText(text="test", bbox=[0, 0, 20, 20])])

        basic_invoice = merge(basic_invoice, pdf_reader_mock.get_text_bbox(file_name))

        assert basic_invoice.kid[0].value == "test"

    def test_should_resize_bbox_if_mismatch_size(self):
        basic_invoice = RawInvoice(kid=[Field(value="est", confidence=0, bbox=[0, 0, 10, 10])])

        pdf_reader_mock = PdfReaderMock()
        file_name = "example"
        pdf_reader_mock.add_file(
            file_name, "test", [BboxText(text="test", bbox=[0, 0, 5, 5], page_width=5, page_height=5)]
        )

        basic_invoice = merge(basic_invoice, pdf_reader_mock.get_text_bbox(file_name), model_page_size=10)

        assert basic_invoice.kid[0].value == "test"
