import io
import json
from typing import List

import fitz
from PIL import Image

from .pdf_reader import BboxText, PdfReader


class PdfReaderMupdf(PdfReader):
    def read(self, file_path: str) -> str:
        doc = fitz.open(file_path)
        text = [page.get_text() for page in doc]
        return "\n".join(text)

    def get_text_bbox(self, file_path: str) -> List[BboxText]:
        doc = fitz.open(file_path)
        page = doc[0]
        tdict = json.loads(page.getText("json"))

        bbox_text = []
        page_to_image = Image.open(io.BytesIO(page.get_pixmap().getImageData("png")))
        w, h = page_to_image.size
        for block in tdict["blocks"]:
            if not "lines" in block:
                continue

            for line in sum(list(map(lambda x: x["spans"], block["lines"])), []):
                bbox_text.append(
                    BboxText(
                        text=line["text"],
                        bbox=line["bbox"],
                        page_width=w,
                        page_height=h,
                        page=0,
                    )
                )
        return bbox_text
