import re

from ..validators.modulus import Modulus
from ..validators.validator import Validator


class AccountNumber(Validator[str]):  # pylint: disable=E1136  # pylint/issues/2822
    def __init__(self, text: str) -> None:
        self.text = text
        self.score = self._calculate_score()

    @staticmethod
    def validate(input_str: str) -> bool:
        if not re.match(r"^[0-9]+$", input_str):
            return False
        account_number = input_str.strip()
        has_correct_length = len(account_number) == 11
        has_correct_check_digit = AccountNumber(input_str).has_correct_check_digit(account_number)
        return has_correct_length and has_correct_check_digit

    def has_correct_check_digit(self, account_number: str) -> bool:
        modulus = Modulus(11, [5, 4, 3, 2, 7, 6, 5, 4, 3, 2])
        check_digit = int(account_number[-1])
        account_number_without_check_digit = account_number[:-1]
        calculated_check_digit = modulus.calculate_check_digit(account_number_without_check_digit)
        return check_digit == calculated_check_digit

    def _calculate_score(self) -> int:
        return 1

    def __str__(self) -> str:
        return "{} : {}".format(self.text, self.score)

    def __repr__(self) -> str:
        return self.__str__()

    def is_valid(self) -> bool:
        return self.validate(self.text)
