import re
from typing import List

from dependency_injector.wiring import Provide, inject

from ..containers.core_container import ContainerRef
from ..pdfreader.pdf_reader import PdfReader
from ..validators.account_number_validator import AccountNumberValidator
from .account_number import AccountNumber
from .text_resolver import TextResolver


class TextAccountNumberFinder(TextResolver):
    @inject
    def __init__(
        self,
        account_number_validator: AccountNumberValidator = Provide[ContainerRef.accountnumber_validator],
        pdf_reader: PdfReader = Provide[ContainerRef.pdf_reader],
    ) -> None:
        self.pdf_reader = pdf_reader
        self.account_number_validator = account_number_validator

    def find_in_file(self, path: str) -> List[str]:
        content = self.pdf_reader.read(path)
        return self.find_in_text(content)

    def find_in_text(self, text: str) -> List[str]:
        numbers_regex = re.compile(r"\d+")
        splitted_numbers = numbers_regex.findall(text)
        joined_chunks = self._join_chunks(splitted_numbers)

        all_accout_numbers = map(AccountNumber, joined_chunks)
        potential_accout_numbers = list(filter(lambda x: x.is_valid(), all_accout_numbers))

        if len(potential_accout_numbers) == 0:
            return []
        return list(map(lambda x: x.text, potential_accout_numbers))

    def _join_chunks(self, numbers):
        candidates = []
        for index, i in enumerate(numbers):
            if (index + 2) < len(numbers):
                candidate_1 = numbers[index] + numbers[index + 1] + numbers[index + 2]
                candidates.append(candidate_1)
            candidates.append(i)
        return candidates
