import sys
import unittest

from ..containers.core_container import Container
from ..pdfreader.pdf_reader_mock import PdfReaderMock
from .text_account_number_finder import TextAccountNumberFinder


class TextAccountNumberFinderTest(unittest.TestCase):
    def setUp(self) -> None:
        Container.wire(modules=[sys.modules[__name__]])

    def test_with_account_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "account_number"
        pdf_reader_mock.add_file(file_name, "account number is here 97752855084 ")
        assert TextAccountNumberFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == ["97752855084"]

    def test_with_noise_account_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "account_number"
        account_numberers_wih_noise = [
            "9775 285 5084",
            "9775.285.5084",
            ("R: 2470.28.89584 OBS!!!", "24702889584"),
            ("sr 20500210940 +» s", "20500210940"),
            (" 6005 06.68447 ", "60050668447"),
        ]
        for account_number_wih_noise in account_numberers_wih_noise:
            pdf_reader_mock.add_file(
                file_name,
                "account number is here {account_number_wih_noise} or ? ".format(
                    account_number_wih_noise=account_number_wih_noise
                    if isinstance(account_number_wih_noise, str)
                    else account_number_wih_noise[0]
                ),
            )
            expected_value = account_number_wih_noise.strip().replace(".", "").replace(" ", "") if isinstance(account_number_wih_noise, str) else account_number_wih_noise[-1]  # type: ignore
            assert TextAccountNumberFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == [
                "{account_number_wih_noise}".format(account_number_wih_noise=expected_value)
            ]

    def test_without_account_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "account_number"
        pdf_reader_mock.add_file(
            file_name,
            "this is a lot of text. Here is a number which is not a account number 97752855032, here is more text",
        )
        assert TextAccountNumberFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == []
