import datetime
import re
from typing import List

from dateutil.parser import parse

from .date import Date
from .text_resolver import TextResolver


def convert_norwegian_months_to_english(string):
    # TODO : this should convert all date string into english shorthand date format
    return string.replace("des", "dec")


def preprocess(string):
    if not isinstance(string, str):
        return string
    # TODO : this should convert all date string into english shorthand date format
    return string.lower().replace("september", "sept").replace(".", " ").strip().replace("  ", " ")


class TextDateFinder(TextResolver):
    def find_in_file(self, path: str) -> List[str]:
        raise Exception("not implemented")

    def find_in_text(self, text: str) -> List[datetime.datetime]:  # type: ignore
        matches = []
        if isinstance(text, datetime.datetime):
            return [text]  # type: ignore
        text = preprocess(text)
        for match in re.findall(r"(\d{1,4}([.\-/])\d{1,2}([.\-/])\d{1,4})", text) + re.findall(r"(\d{1,4}([.\-\/ ])\w{1,4}([.\-\/ ])\d{1,4})", text):  # type: ignore [type-var]
            for date in match:
                date = convert_norwegian_months_to_english(date)
                if Date.validate(date):
                    matches.append(parse(date))
        return list(set(matches))
