import re
from typing import List

from dependency_injector.wiring import Provide, inject
from pdfinvoicereader.validators.account_number_validator import AccountNumberValidator

from ..containers.core_container import ContainerRef
from ..pdfreader.pdf_reader import PdfReader
from .kid import Kid
from .text_resolver import TextResolver


class TextKidFinder(TextResolver):
    @inject
    def __init__(
        self,
        account_number_validator: AccountNumberValidator = Provide[ContainerRef.accountnumber_validator],
        pdf_reader: PdfReader = Provide[ContainerRef.pdf_reader],
    ) -> None:
        self.pdf_reader = pdf_reader
        self.account_number_validator = account_number_validator

    def find_in_file(self, path: str) -> List[str]:
        content = self.pdf_reader.read(path)
        return self.find_in_text(content)

    def find_in_text(self, text: str) -> List[str]:
        numbers_regex = re.compile(r"\d+")
        splitted_numbers = numbers_regex.findall(text)
        joined_chuncks = self._join_chunks(splitted_numbers)

        all_kids = map(Kid, joined_chuncks)
        potential_kids = list(filter(lambda x: x.is_valid(), all_kids))

        if len(potential_kids) == 0:
            return []
        best_scored_kid = sorted(potential_kids, key=lambda x: x.score)[-1]
        return [best_scored_kid.text]

    def _join_chunks(self, numbers):
        candidates = []
        for index, i in enumerate(numbers):
            if (index + 1) < len(numbers):
                candidate_1 = numbers[index] + numbers[index + 1]
                candidates.append(candidate_1)
            candidates.append(i)
        return candidates
