import sys
import unittest

from ..containers.core_container import Container
from ..pdfreader.pdf_reader_mock import PdfReaderMock
from .text_kid_finder import TextKidFinder


class TextKidFinderTest(unittest.TestCase):
    def setUp(self) -> None:
        Container.wire(modules=[sys.modules[__name__]])

    def test_with_kid(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "kid"
        pdf_reader_mock.add_file(file_name, "this is a lot of text. Here is a kid 2345676, here is more text")
        assert TextKidFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == ["2345676"]

    def test_without_kid(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "kid"
        pdf_reader_mock.add_file(
            file_name,
            "this is a lot of text. Here is a number which is not a kid 12213, here is more text",
        )
        assert TextKidFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == []

    def test_with_splitted_kid(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        file_name = "kid"
        pdf_reader_mock.add_file(
            file_name,
            "this is a lot of text. Here is a number which is not a kid 52134201 10636, here is more text",
        )
        assert TextKidFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == ["5213420110636"]
