import re
from typing import List

from dependency_injector.wiring import Provide, inject

from ..containers.core_container import ContainerRef
from ..pdfreader.pdf_reader import PdfReader
from ..validators.account_number_validator import AccountNumberValidator
from .number_field import NumberField
from .text_resolver import TextResolver


class TextNumberFieldFinder(TextResolver):
    @inject
    def __init__(
        self,
        account_number_validator: AccountNumberValidator = Provide[ContainerRef.accountnumber_validator],
        pdf_reader: PdfReader = Provide[ContainerRef.pdf_reader],
    ) -> None:
        self.pdf_reader = pdf_reader
        self.account_number_validator = account_number_validator

    def find_in_file(self, path: str) -> List[str]:
        content = self.pdf_reader.read(path)
        return self.find_in_text(content)

    def find_in_text(self, text: str) -> List[str]:
        numbers_regex = re.compile(r"\d+")
        splitted_numbers = numbers_regex.findall(text)
        all_number_fields = map(NumberField, splitted_numbers)

        potential_numbers = list(filter(lambda x: x.is_valid(), all_number_fields))

        if len(potential_numbers) == 0:
            return []
        return list(map(lambda x: x.text, potential_numbers))
