from typing import List

from dependency_injector.wiring import Provide, inject

from ..containers.core_container import ContainerRef
from ..pdfreader.pdf_reader import PdfReader
from .principal_amount import PrincipalAmount
from .text_resolver import TextResolver


class TextPrincipalAmountFinder(TextResolver):
    @inject
    def __init__(
        self,
        pdf_reader: PdfReader = Provide[ContainerRef.pdf_reader],
    ) -> None:
        self.pdf_reader = pdf_reader

    def find_in_file(self, path: str) -> List[str]:
        content = self.pdf_reader.read(path)
        return self.find_in_text(content)

    def find_in_text(self, text: str) -> List[str]:
        if not self._has_space_after_comma(text):
            principal_amount_field = PrincipalAmount(self._preprocess(text))
            if principal_amount_field.is_valid():
                return [self._preprocess(principal_amount_field.text)]
        for i in self._split(text):
            preprocessed_text = self._preprocess(i)
            if "," in preprocessed_text:
                principal_amount_field = PrincipalAmount(preprocessed_text)
                if principal_amount_field.is_valid():
                    return [self._preprocess(preprocessed_text)]
        return []

    def _preprocess(self, text):
        # TODO : add regex to check this instead
        if "." in text and "," in text:  # 80.000,00
            text = text.replace(".", "")
        if text.count(".") > 1 and len(text.split(".")[-1]) == 2:  # 80.000,00
            text = text.split(".")
            text = "".join(text[:-1]) + "," + text[-1]
        return text.replace("\x0c", "").strip()

    def _has_space_after_comma(self, text):
        if " " in text and "," in text:
            return text.index(",") < text.index(" ")
        return "\n" in text

    def _split(self, text):
        split_new_line = text.split("\n")
        return sum([i.split(" ") for i in split_new_line], [])
