import sys
import unittest

from ..containers.core_container import Container
from ..pdfreader.pdf_reader_mock import PdfReaderMock
from .text_principal_amount_finder import TextPrincipalAmountFinder


class TextPrincipalAmountFinderTest(unittest.TestCase):
    def setUp(self) -> None:
        Container.wire(modules=[sys.modules[__name__]])

    def test_with_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        for valid_amounts, expected in zip(
            [
                "123,00",
                "123 435, 00",
                "2 768,00       \n",
                "500\x0c",
                "NOK 1760,00\n1902",
                "this is a lot of text NOK 1760,00",
                "10.973,00 29016511590",
                "5.933.00",
                "5.933.00\n\x0c",
            ],
            ["123,00", "123 435, 00", "2 768,00", "500", "1760,00", "1760,00", "10973,00", "5933,00", "5933,00"],
        ):
            file_name = "number"
            pdf_reader_mock.add_file(file_name, valid_amounts)
            assert TextPrincipalAmountFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == [expected]

    def test_without_number(self) -> None:
        pdf_reader_mock = PdfReaderMock()
        for invalid_amounts in [
            " 123.00 more text",
        ]:
            file_name = "number"
            pdf_reader_mock.add_file(
                file_name,
                invalid_amounts,
            )
            assert TextPrincipalAmountFinder(pdf_reader=pdf_reader_mock).find_in_file(file_name) == []
