import glob
import json
import os
from collections import defaultdict

from Levenshtein import distance
from pdf_invoice_reader_common.helper import json_flaten
from pdfinvoicereader.parse_raw_document import parse_document
from testing_model_interface import convert_to_json
from yolov3.detect_interface import Detect


def get_grouped_invoices():
    GROUPED_INVOICE_DESIGN_PATH = (
        "/home/brage/sambashare/pdf-invoice-reader/projects/pdf_invoice_reader/datasets/scripts/grouped-invoices/"
    )
    GROUPED_INVOICE_ONE_PER_CLASS = defaultdict(list)
    for i in glob.glob(os.path.join(GROUPED_INVOICE_DESIGN_PATH, "**/*.png")):
        CLASS_ID = os.path.dirname(i).split("/")[-1]
        GROUPED_INVOICE_ONE_PER_CLASS[CLASS_ID].append(i)

    return GROUPED_INVOICE_ONE_PER_CLASS


def validate(predicted, originale):
    score = {}
    for key, value in predicted[0].items():
        camelCase = key
        if "account" in camelCase:
            camelCase = camelCase.replace("invoice", "invoicing")
        if "principal_amount" in camelCase and value is not None:
            value = value.replace(" ", "").replace(",", ".")
        if camelCase.count("_") >= 1:
            camelCase = camelCase.split("_")
            camelCase = camelCase[0] + "".join([(i[0].upper() + i[1:]) for i in camelCase[1:]])
        if camelCase not in originale:
            return None
        print(key, value, originale[camelCase])
        if value is None and originale[camelCase] is not None:
            score[key] = {
                "found_none_but_was_defined": value is None,
            }
        else:
            correct = int(str(value) == str(originale[camelCase]))
            if correct:
                score[key] = {
                    "correct": correct,
                }
            elif originale[camelCase] is not None:
                score[key] = {
                    "distance": distance(value, originale[camelCase]),
                    "distance_count": 1,
                    "correct": 1 if distance(value, originale[camelCase]) < 3 else 0,
                }
            else:
                score[key] = {
                    "misclassified_as_not_none": 1,
                }
    return score


def get_accuracy(model, GROUPED_INVOICE_ONE_PER_CLASS):
    global_score = defaultdict(dict)
    COUNT = 0
    for key in GROUPED_INVOICE_ONE_PER_CLASS:
        for value in GROUPED_INVOICE_ONE_PER_CLASS[key]:
            dir_name = "/".join(os.path.basename(value).split("_")[:2]) + ".json"
            json_path = os.path.join(
                "/home/brage/sambashare/pdf-invoice-reader/projects/pdf_invoice_reader/datasets/jsons/", dir_name
            )
            print(key, value)
            _, clean_document = parse_document(
                model.get_document_from_path(value, prefix=model.prefix, debug=False, plot=False)
            )
            predicted_json = json.loads(convert_to_json(clean_document))
            with open(json_path, "r") as file:
                print(json_path)
                originale_json = json_flaten(json.loads(file.read()))

            score_dict = validate(predicted_json, originale_json)
            if score_dict is None:
                continue
            for field, field_scores in score_dict.items():
                for metric, score in field_scores.items():
                    if metric not in global_score[field]:
                        global_score[field][metric] = 0
                    global_score[field][metric] += score
            COUNT += 1
            break

    print(global_score)
    print(COUNT)
    return sum([i.get("correct", 0) for i in global_score.values()]) / (COUNT * len(model.classes))


if __name__ == "__main__":
    model = Detect(data_config=None, prefix="invoice_model_v2")
    GROUPED_INVOICE_ONE_PER_CLASS = get_grouped_invoices()
    print(get_accuracy(model, GROUPED_INVOICE_ONE_PER_CLASS))

# 0.5795918367346938
