import os

import tqdm
from torch.autograd import Variable

from .train import get_model, get_opts_raw, get_train_and_validate_dataset, load_filters
from .utils.parse_config import *
from .utils.transforms import *
from .utils.utils import *

parser = get_opts_raw()
parser.add_argument("--save", type=str, default="", help="save the output to file")
opt = parser.parse_args()

data_config = parse_data_config(opt.data_config)
current_dir = os.path.dirname(os.path.abspath(__file__))
root_path = os.path.join(current_dir, data_config["root_folder"])
image_path = os.path.join(root_path, "images")
class_names = os.path.join(root_path, "classes.names")
filters = load_filters(data_config.get("filter", None), current_dir)

device = torch.device("cuda")
model = get_model(opt, device)

dataset_train, dataset_validator = get_train_and_validate_dataset(
    image_path=image_path,
    split=opt.datset_split,
    opt=opt,
    transform=DEFAULT_TRANSFORMS,
    max_files=100,
    filters=filters,
)

dataloader = torch.utils.data.DataLoader(
    dataset_train,
    batch_size=1,
    shuffle=True,
    num_workers=2,
    pin_memory=True,
    collate_fn=dataset_train.collate_fn,
)
mapping = {}
model.eval()
for batch_i, (file_img, imgs, targets) in enumerate(tqdm.tqdm(dataloader)):
    imgs = Variable(imgs.to(device))
    targets = Variable(targets.to(device), requires_grad=False)

    loss, outputs = model(imgs, targets)
    # print(loss)
    # print(file_img)
    mapping[file_img[0]] = loss.item()

if opt.save:
    print("saved to ", opt.save)
    creditor_ids = []
    for key, value in (sorted(mapping.items(), key=lambda x: x[1])[::-1])[:25]:
        creditor_ids.append(os.path.basename(key).split("_")[0])
    with open(opt.save, "w") as file:
        for i in list(set(creditor_ids)):
            file.write(i + "\n")
else:
    for key, value in (sorted(mapping.items(), key=lambda x: x[1])[::-1])[:25]:
        print("{} {}".format(key, value))
